@@ -10,9 +10,14 @@ test_that("qualpal() returns the proper list object", {
   })
 
 test_that("erroneous input to qualpal() returns errors", {
-  expect_error(qualpal("colorblind"))
+  expect_error(qualpal("cvd"))
+  expect_error(qualpal(2, severity = 1.5))
+  expect_error(qualpal(3, severity = -5))
+  expect_error(qualpal(5, cvd = "normal"))
+  expect_warning(qualpal(5, colorblind = "deutan"))
+  expect_error(qualpal(1))
   expect_error(qualpal(n = 0))
-  expect_error(qualpal(n = 2, colorblind = "deutrenop"))
+  expect_error(qualpal(n = 2, cvd = "deutrenop"))
   expect_error(qualpal(n = 10 ^ 3))
   expect_error(qualpal(n = 2, "prety"))
   expect_error(qualpal(n = 3, colorspace = c(0, 200)))
@@ -21,7 +26,7 @@ test_that("erroneous input to qualpal() returns errors", {
   expect_error(qualpal(n = 2, list(h = c(-200, 200), s = c(0, 1), l = c(0, 1))))
   expect_error(qualpal(n = 2, list(h = c(0, 200), s = c(0, 1))))
   expect_error(qualpal(n = 2, list(h = 2, s = c(0, 1), l = c(0, 1))))
-  expect_warning(qualpal(n = 2, "pretty", neval = 5000))
+  expect_error(qualpal(n = 500))
 })
 
 test_that("plots return errors when used inappropriately", {
